% variables locales
%
% length_vec_sigma est la taille du vecteur contenant les differentes
% valeurs de sigma
length_vec_sigma=500;
%
vec_x=linspace(0,0.5,length_vec_sigma);
vec_sigma=vec_x*sqrt(N)./(1-vec_x);
%
% vec_taux sont les matrices qui contiennent les taux de classification
M=1;m=1;
vec_taux_app_Hebb=zeros(length_vec_sigma,M);
vec_taux_gen_Hebb=zeros(length_vec_sigma,M);
vec_taux_app_PI=zeros(length_vec_sigma,M);
vec_taux_gen_PI=zeros(length_vec_sigma,M);
vec_taux_app_RA=zeros(length_vec_sigma,M);
vec_taux_gen_RA=zeros(length_vec_sigma,M);
%
% w_vrai est le vecteur frontiere de la base d'apprentissage lineairement separable
w_vrai=randn(N,1);
%
h_bar = waitbar(0,'Please wait...');
%
for ind_sigma=1:length(vec_sigma)
    waitbar(ind_sigma/length(vec_sigma),h_bar)
    %
    sigma=vec_sigma(ind_sigma);
    %
    % generation de la base d'apprentissage et de la base de gen
    if ind_sigma==1
        [Xapp,dapp,Xgen,dgen]=genere_base_lineairement_separable(w_vrai,Papp,Pgen);
    elseif (choix_nouvelle_base_app==1)&&(choix_nouvelle_base_gen==1)
        [Xapp,dapp,Xgen,dgen]=genere_base_lineairement_separable(w_vrai,Papp,Pgen);
    elseif (choix_nouvelle_base_app==1)&&(choix_nouvelle_base_gen==0)
        [Xapp,dapp]=genere_base_lineairement_separable(w_vrai,Papp,Pgen);
    elseif (choix_nouvelle_base_app==0)&&(choix_nouvelle_base_gen==1)
        [~,~,Xgen,dgen]=genere_base_lineairement_separable(w_vrai,Papp,Pgen);
    end
    %
    % Hebb
    %
    w=discriminateur_Hebb(Xapp,dapp);
    %
    d_est=sign(real(Xapp'*w));
    taux_app=mean(d_est==dapp);
    %
    d_est=sign(real(Xgen'*w));
    taux_gen=mean(d_est==dgen);
    %
    vec_taux_app_Hebb(ind_sigma,m)=taux_app;
    vec_taux_gen_Hebb(ind_sigma,m)=taux_gen;
    %
    % Pseudo-inverse
    %
    w=discriminateur_PI(Xapp,dapp);
    %
    d_est=sign(real(Xapp'*w));
    taux_app=mean(d_est==dapp);
    %
    d_est=sign(real(Xgen'*w));
    taux_gen=mean(d_est==dgen);
    %
    vec_taux_app_PI(ind_sigma,m)=taux_app;
    vec_taux_gen_PI(ind_sigma,m)=taux_gen;
    %
    % ridge approximation
    %
    w=discriminateur_RA(Xapp,dapp,sigma);
    %
    d_est=sign(real(Xapp'*w));
    taux_app=mean(d_est==dapp);
    %
    d_est=sign(real(Xgen'*w));
    taux_gen=mean(d_est==dgen);
    %
    vec_taux_app_RA(ind_sigma,m)=taux_app;
    vec_taux_gen_RA(ind_sigma,m)=taux_gen;
end
%
close(h_bar)
%
% calcul et plot des taux moyen
%
figure
cell_couleur={'b','r','g'};
cell_forme={'x','o','+'};
h=zeros(6,1);
for methode=1:3
    %
    couleur=cell_couleur{methode};
    forme=cell_forme{methode};
    %
    if methode==1
        vec_taux_app=vec_taux_app_Hebb;
        vec_taux_gen=vec_taux_gen_Hebb;
    elseif methode==2
        vec_taux_app=vec_taux_app_PI;
        vec_taux_gen=vec_taux_gen_PI;
    else
        vec_taux_app=vec_taux_app_RA;
        vec_taux_gen=vec_taux_gen_RA;
    end
    % base d'apprentissage
    vec_mean_taux_app=mean(vec_taux_app,2);
    vec_std_taux_app=std(vec_taux_app,0,2);
    %
    % base de gen
    vec_mean_taux_gen=mean(vec_taux_gen,2);
    vec_std_taux_gen=std(vec_taux_gen,0,2);
    %
    % plot
    subplot(2,1,1)
    if M==1
        plot(vec_x,vec_mean_taux_app,[forme couleur])
    else
        errorbar(vec_x,vec_mean_taux_app,vec_std_taux_app,couleur);
    end
    %
    if methode==1
        hold on
        title(['taux de reussite obtenu sur la base d''apprentissage' ...
            ', P_{app} = ' num2str(Papp)])
        set(gca,'fontsize',14)
    end
    %
    subplot(2,1,2)
    if M==1
        plot(vec_x,vec_mean_taux_gen,[forme couleur])
    else
        errorbar(vec_x,vec_mean_taux_gen,vec_std_taux_gen,couleur);
    end
    %
    if methode==1
        hold on
        title(['taux de generalisation' ...
            ', P_{gen} = ' num2str(Pgen)])
        set(gca,'fontsize',14)
    end
end
%
for m=1:2
    subplot(2,1,m)
    ylabel('taux de reussite')
    grid on
    xlabel('\sigma/(\sigma+sqrt(N))')
    %
    if m==1
        set(gca,'ylim',[0.8 1.1])
        ylabel('\tau_{app}')
    else
        set(gca,'ylim',[0.4 0.9])
        ylabel('\tau_{g}')
    end
    set(gca,'xlim',[vec_x(1) vec_x(end)])
    %
    L=legend('Hebb','PI','RA');
    set(L,'Location','South')
end
