function [Xapp,dapp,Xtest,dtest]=genere_base_non_lineairement_separable(N,Papp,Ptest)
%
% [Xapp,dapp,Xtest,dtest]=genere_base_non_lineairement_separable(N,Papp,Ptest);
%
% entrees
%========
% N : dimension des vecteurs X
% Papp : taille de la base d'apprentissage
% Ptest : taille de la base de test
%
% sorties
%========
% Xapp : matrice de taille NxPapp 
% dapp : vecteur colonne de taille Papp
% Xtest : matrice de taille NxPtest 
% dtest : vecteur colonne de taille Ptest
%
% ou
%
% [Xapp,dapp,Xtest,dtest]=genere_base_non_lineairement_separable(Xapp,dapp,Ptest);
%
% entrees
%========
% N : dimension des vecteurs X
% Papp : taille de la base d'apprentissage
% Ptest : taille de la base de test
%
% sorties
%========
% Xapp : matrice de taille NxPapp 
% dapp : vecteur colonne de taille Papp
% Xtest : matrice de taille NxPtest 
% dtest : vecteur colonne de taille Ptest

if isscalar(N)
    % generation de la base d'apprentissage
    %
    % generation de 2.Papp vecteur aleatoires
    Xapp=[randn(N,Papp) zeros(N,Papp)];
    %
    % on assigne la classe 1 ou 2 avec une probabilite 1/2
    dapp=[sign(rand(Papp,1)-0.5);zeros(Papp,1)];
    %
    % traitement des vecteurs de Papp+1 a 2.Papp
    ind_p=1:Papp;
    Xapp(:,ind_p+Papp)=-Xapp(:,ind_p);
    dapp(ind_p+Papp)=-dapp(ind_p);
else
    Xapp=N;
    dapp=Papp;
    N=size(Xapp,1);
end
%
if (nargin==2)
    return
end
%
% generation de la base de test
%
% generation de Ptest vecteur aleatoires
Xtest=randn(N,Ptest);
Papp=size(Xapp,2);
%
% calcul de la classe de ces vecteurs
dtest=zeros(Ptest,1);
%
for p=1:Ptest
    % calcul de la distance aux vecteurs de la base d'apprentissage
    Y=Xtest(:,p)*ones(1,Papp)-Xapp;
    vec=sum(Y.^2);
    %
    % on recherche le vecteur le plus proche
    ind=find(vec==min(vec));
    ind=ind(1);
    %
    % on attirbue la classe du plus proche
    dtest(p)=dapp(ind);
end
